import os
import mysql.connector
import pandas as pd
from configparser import NoSectionError
from pathlib import Path
from mysql.connector.errors import DatabaseError, OperationalError, InternalError
from retrying import retry
from sroka.api.mysql.mysql_helpers import validate_options, get_options_from_config


@retry(stop_max_attempt_number=1,
       wait_exponential_multiplier=1 * 2,
       wait_exponential_max=1 * 2 * 2)
def query_mysql(query: str, filename=None,
                host=None, port=None,
                unix_socket=None, user=None,
                password=None, database=None
                ):
    try:
        options = get_options_from_config()

    except NoSectionError:
        print('Missing MySQL section in configuration')
        return pd.DataFrame([])

    if host:
        options['host'] = host
    if port:
        options['port'] = port
    if unix_socket:
        options['unix_socket'] = unix_socket
    if user:
        options['user'] = user
    if password:
        options['password'] = password
    if database:
        options['database'] = database

    if not validate_options(options):
        return pd.DataFrame([])

    try:
        # Connect while passing only the arguments that were set in the
        # configuration, since some are mutually exclusive or optional.
        connection = mysql.connector.connect(**{k: v for k, v in options.items() if v != ""})

        # Get the MySQL connector cursor, which allows interacting with the
        # MySQL server.
        cursor = connection.cursor()

        # Execute the query.
        cursor.execute(query)

    except OperationalError as e:
        print('Operational MySQL Error: {}'.format(e))
        return pd.DataFrame([])
    except InternalError as e:
        print('Internal MySQL Error: {}'.format(e))
        return pd.DataFrame([])
    except DatabaseError as e:
        print('Database MySQL Error: {}'.format(e))
        return pd.DataFrame([])

    # Cycle through the returned elements to build a pandas DataFrame.
    df = pd.DataFrame(cursor, columns=cursor.column_names)

    # Close the connection to the MySQL server.
    cursor.close()
    connection.close()

    # If no filename is specified, return the data as a pandas Dataframe.
    # Otherwise, store it in a file.
    if not filename:
        return df

    # Store the path in a cross-platform pathlib object to ensure compatibility
    # with DOS & UNIX-based operating systems.
    path = Path(filename)

    # Get the parent directory of the given path, if it exists.
    directory_path = str(path.parent.resolve())

    # If the given path points to a folder, attempt to create it. If it already
    # exists, the `exist_ok` option ensures that no exception will be thrown.
    if directory_path != "":
        os.makedirs(directory_path, exist_ok=True)

    # Export the data in a CSV file.
    try:
        df.to_csv(filename)
    except OSError as e:
        print('Unable to write on filesystem: {}'.format(e))



import urllib.parse
import urllib.request
from configparser import NoOptionError
from io import StringIO

import pandas as pd

import sroka.config.config as config


def get_data_from_rubicon(rubicon_dict, currency='USD'):
    """
    Function that download data from Rubicon db through API to pandas DataFrame.

    Args:
        rubicon_dict (dict):  dictionary with keys:
            'start' (str : str) (Start date in ISO-8601 format, including time zone.) - obligatory,
            'end' (str : str) (End date in ISO-8601 format, including time zone.) - obligatory,
            'dimensions' (str : list of str) (dimensions that we want included as columns in list) - obligatory,
            'metrics' (str : list of str) (metrics that we want included as columns in list) - obligatory
            'filters' (str: list of str) (filters that we want to be included in list) - obligatory
        currency (str): currency to be used

    Returns:
        pandas DataFrame

    Full documentation (additional features) of Rubicon API is available at:
        https://resources.rubiconproject.com/resource/publisher-resources/performance-analytics-api/
        (you need to be logged in with provided credentials)
    """

    try:
        username = config.get_value('rubicon', 'username')
        password = config.get_value('rubicon', 'password')
        rubicon_id = config.get_value('rubicon', 'id')
    except (KeyError, NoOptionError):
        print('No credentials were provided')
        return pd.DataFrame([])

    if 'start' not in rubicon_dict.keys() or 'end' not in rubicon_dict.keys():
        print('Required fields are not set')
        return pd.DataFrame([])

    rubicon_dict['currency'] = currency

    try:
        if not rubicon_dict['dimensions'] or not rubicon_dict['metrics'] or not rubicon_dict['filters']:
            print('Required fields are empty')
            return pd.DataFrame([])
        rubicon_dict['dimensions'] = ','.join(rubicon_dict['dimensions'])
        rubicon_dict['metrics'] = ','.join(rubicon_dict['metrics'])
        rubicon_dict['filters'] = ';'.join(rubicon_dict['filters'])
    except KeyError:
        print('Required fields are not set')
        return pd.DataFrame([])

    url = urllib.parse.unquote(
        'https://api.rubiconproject.com/analytics/v1/report/?account=publisher/{}&'.format(
            rubicon_id) + urllib.parse.urlencode(rubicon_dict))

    p = urllib.request.HTTPPasswordMgrWithDefaultRealm()
    p.add_password(None, url, username, password)

    auth_handler = urllib.request.HTTPBasicAuthHandler(p)

    opener = urllib.request.build_opener(auth_handler)

    opener.addheaders = [('Accept', 'text/csv')]

    urllib.request.install_opener(opener)

    try:
        result = opener.open(url)

    except IOError as e:
        print('Something went wrong, please see error message:')
        print(e)
        return pd.DataFrame([])

    result_data = result.read().decode()
    df = pd.read_csv(StringIO(result_data))
    return df



import gzip
import json
import tempfile
from configparser import NoOptionError

import pandas as pd
from googleads import ad_manager, errors
from zeep import helpers

import sroka.config.config as config

# import variable_validators as validator

KEY_FILE = config.get_file_path('google_ad_manager')

# GAM API information.
try:
    APPLICATION_NAME = config.get_value('google_ad_manager', 'application_name')
except (KeyError, NoOptionError):
    APPLICATION_NAME = 'Application name'


def dict_type_checker(dict_argument, argument_name, mandatory=True):
    if mandatory:
        if type(dict_argument) != dict:
            print(f"""{argument_name} needs to be a dict""")
            return "Incorrect type"
    else:
        if dict_argument and type(dict_argument) != dict:
            print(f"""{argument_name} needs to be a dict""")
            return "Incorrect type"


def int_type_checker(int_argument, argument_name, mandatory=True):
    if mandatory:
        if type(int_argument) != int:
            print(f"""{argument_name} needs to be an integer""")
            return "Incorrect type"
    else:
        if int_argument and type(int_argument) != int:
            print(f"""{argument_name} needs to be an integer""")
            return "Incorrect type"


def list_type_checker(list_argument, argument_name, mandatory=True):
    if mandatory:
        if type(list_argument) != list:
            print(f"""{argument_name} needs to be a list""")
            return "Incorrect type"
    else:
        if list_argument and type(list_argument) != list:
            print(f"""{argument_name} needs to be a list""")
            return "Incorrect type"


def str_type_checker(str_argument, argument_name, mandatory=True):
    if mandatory:
        if type(str_argument) != str:
            print(f"""{argument_name} needs to be a string""")
            return "Incorrect type"
    else:
        if str_argument and type(str_argument) != str:
            print(f"""{argument_name} needs to be a string""")
            return "Incorrect type"


def init_gam_connection(network_code=None):
    if not network_code:
        try:
            network_code = config.get_value('google_ad_manager', 'network_code')
        except (KeyError, NoOptionError):
            print('No network code was provided')
            return pd.DataFrame([])
    yaml_string = "ad_manager: " + "\n" + \
                  "  application_name: " + APPLICATION_NAME + "\n" + \
                  "  network_code: " + str(network_code) + "\n" + \
                  "  path_to_private_key_file: " + KEY_FILE + "\n"

    # Initialize the GAM client.
    gam_client = ad_manager.AdManagerClient.LoadFromString(yaml_string)
    return gam_client


def get_data_from_admanager(query, dimensions, columns, start_date, end_date, custom_field_id=None,
                            dimension_attributes=None, network_code=None):
    if not custom_field_id:
        custom_field_id = []

    if not dimension_attributes:
        dimension_attributes = []

    list_of_types = [str_type_checker(query, "query"),
                     list_type_checker(dimensions, "dimensions"),
                     list_type_checker(columns, "columns"),
                     dict_type_checker(start_date, "start_date"),
                     dict_type_checker(end_date, "end_date"),
                     list_type_checker(custom_field_id, "custom_field_id", False),
                     list_type_checker(dimension_attributes, "dimmension_attributes", False),
                     int_type_checker(network_code, "network_code", False)]

    if "Incorrect type" in list_of_types:
        return

    gam_client = init_gam_connection(network_code)

    # Create statement object to filter for an order.

    filter_statement = {'query': query}

    # Create report job.
    report_job = {
        'reportQuery': {
            'dimensions': dimensions,
            'statement': filter_statement,
            'columns': columns,
            'customFieldIds': custom_field_id,
            'dimensionAttributes': dimension_attributes,
            'dateRangeType': 'CUSTOM_DATE',
            'startDate': start_date,
            'endDate': end_date,
            'adUnitView': "FLAT"
        }
    }

    report_downloader = gam_client.GetDataDownloader()

    try:
        # Run the report and wait for it to finish.
        report_job_id = report_downloader.WaitForReport(report_job)
    except errors.GoogleAdsServerFault as e:
        if 'AuthenticationError.NETWORK_NOT_FOUND' in str(e):
            print('Provided network code was not found.')
        elif 'AuthenticationError.NETWORK_CODE_REQUIRED' in str(e):
            print('Default value of network code is missing from ', config.default_config_filepath)
        else:
            print('Failed to generate report. Error was: {}'.format(e))
        return

    except errors.AdManagerReportError as e:
        print('Failed to generate report. Error was: {}'.format(e))
        return

    export_format = 'CSV_DUMP'

    report_file = tempfile.NamedTemporaryFile(suffix='.csv.gz', delete=False)

    # Download report data.
    report_downloader.DownloadReportToFile(
        report_job_id, export_format, report_file, use_gzip_compression=True)

    report_file.close()

    # Display results.
    print('Report job with id {} downloaded to:\n{}'.format(
        report_job_id, report_file.name))
    with gzip.open(report_file.name) as file:
        data = pd.read_csv(file)
    return data


def get_users_from_admanager(query, dimensions, network_code=None):
    list_of_types = [str_type_checker(query, "query"),
                     list_type_checker(dimensions, "dimensions"),
                     int_type_checker(network_code, "network_code", False)]

    if "Incorrect type" in list_of_types:
        return

    user_df = pd.DataFrame()

    dimensions_df = pd.DataFrame()

    # .Where() handles filtering with admanager method, while statement_query handles queries with WHERE clause

    statement_query = query.upper().replace("WHERE ", "")

    statement = ad_manager.StatementBuilder().Where(statement_query)

    gam_client = init_gam_connection(network_code)

    user_service = gam_client.GetService('UserService')

    try:
        while True:
            response = user_service.getUsersByStatement(statement.ToStatement())
            if 'results' in response and len(response['results']):
                for user in response['results']:

                    for dimension in dimensions:
                        try:
                            dimension_value = [user[dimension]]
                            dimensions_df[dimension] = dimension_value
                        except KeyError as e:
                            print('Failed to generate user list. Incorrect dimension: {}'.format(e))
                            return

                    user_df = user_df.append(dimensions_df, sort=False)
                statement.offset += statement.limit
            else:
                break
        return user_df

    except errors.GoogleAdsServerFault as e:
        if 'AuthenticationError.NETWORK_NOT_FOUND' in str(e):
            print('Provided network code was not found.')
        elif 'AuthenticationError.NETWORK_CODE_REQUIRED' in str(e):
            print('Default value of network code is missing from ', config.default_config_filepath)
        else:
            print('Failed to generate user list. Error was: {}'.format(e))
        return

    except errors.AdManagerReportError as e:
        print('Failed to generate user list. Error was: {}'.format(e))
        return


def get_companies_from_admanager(query, dimensions, network_code=None):
    list_of_types = [str_type_checker(query, "query"),
                     list_type_checker(dimensions, "dimensions"),
                     int_type_checker(network_code, "network_code", False)]

    if "Incorrect type" in list_of_types:
        return

    company_df = pd.DataFrame()

    dimensions_df = pd.DataFrame()

    # .Where() handles filtering with admanager method, while statement_query handles queries with WHERE clause

    statement_query = query.upper().replace("WHERE ", "")

    statement = ad_manager.StatementBuilder().Where(statement_query)

    gam_client = init_gam_connection(network_code)

    company_service = gam_client.GetService('CompanyService')

    try:
        while True:
            response = company_service.getCompaniesByStatement(statement.ToStatement())
            if 'results' in response and len(response['results']):
                for company in response['results']:

                    for dimension in dimensions:
                        try:
                            dimension_value = [company[dimension]]
                            dimensions_df[dimension] = dimension_value
                        except KeyError as e:
                            print('Failed to generate company list. Incorrect dimension: {}'.format(e))
                            return

                    company_df = company_df.append(dimensions_df, sort=False)
                statement.offset += statement.limit
            else:
                break
        return company_df

    except errors.GoogleAdsServerFault as e:
        if 'AuthenticationError.NETWORK_NOT_FOUND' in str(e):
            print('Provided network code was not found.')
        elif 'AuthenticationError.NETWORK_CODE_REQUIRED' in str(e):
            print('Default value of network code is missing from ', config.default_config_filepath)
        else:
            print('Failed to generate company list. Error was: {}'.format(e))
        return

    except errors.AdManagerReportError as e:
        print('Failed to generate company list. Error was: {}'.format(e))
        return


def serialize_gam_object(obj: object, columns_to_keep: list[str] = None) -> dict:
    """
    Serializes a zeep object from the GAM API into a flattened dictionary,
    handling nested objects and lists.

    Args:
        obj: The zeep object to serialize.
        columns_to_keep: An optional list of column names to keep. If provided,
                         only these columns will be in the output dict.

    Returns:
        A flattened dictionary representation of the object.
    """
    base_dict = helpers.serialize_object(obj, dict)

    def flatten_dict(d: dict, parent_key: str = '', sep: str = '_') -> dict:
        items = []
        for k, v in d.items():
            new_key = parent_key + sep + k if parent_key else k
            if isinstance(v, dict):
                items.extend(flatten_dict(v, new_key, sep=sep).items())
            elif isinstance(v, list):
                if v and all(isinstance(i, dict) for i in v):
                    items.append((new_key, json.dumps(v)))
                else:
                    items.append((new_key, ','.join(map(str, v))))
            else:
                items.append((new_key, v))
        return dict(items)

    flattened_data = flatten_dict(base_dict)

    if columns_to_keep:
        return {key: flattened_data.get(key) for key in columns_to_keep}

    return flattened_data


def get_service_data_from_admanager(
    service: str,
    query_filter: str = None,
    columns_to_keep: list[str] = None,
    network_code: str = None,
) -> pd.DataFrame:
    """
    Fetches a complete list of a specified service data type from Google Ad Manager.

    This generic function uses the appropriate service (e.g., InventoryService,
    LineItemService). It handles pagination
    automatically to retrieve all entities matching the query.

    Args:
        service: The type of service data to fetch. Must be a key in the service_map (e.g., 'AdUnit').
        query_filter: An optional PQL-like 'WHERE' clause to filter the results.
                     For example: "WHERE status = 'ACTIVE'". Do not include
                     'ORDER BY' or 'LIMIT' clauses.
        columns_to_keep: An optional list of column names to keep in the output DataFrame.
                    If None, provides all the columns.
        network_code: The GAM network code to use.
    Returns:
        A pandas DataFrame with all the items in the specified service data.

    Raises:
        ValueError: If the provided service is not supported.
        Exception: Propagates exceptions from the GAM API client.
    """

    gam_api_page_limit = 500
    service_map = {
        "AdUnit": ("InventoryService", "getAdUnitsByStatement"),
        "LineItem": ("LineItemService", "getLineItemsByStatement"),
        "Order": ("OrderService", "getOrdersByStatement"),
        "Creative": ("CreativeService", "getCreativesByStatement"),
        "Company": ("CompanyService", "getCompaniesByStatement"),
        "Label": ("LabelService", "getLabelsByStatement"),
        "CustomField": ("CustomFieldService", "getCustomFieldsByStatement"),
    }

    if service not in service_map:
        raise ValueError(
            f"Unsupported inventory_type: '{service}'. "
            f"Supported types are: {list(service_map.keys())}"
        )

    service_name, method_name = service_map[service]
    print(f"Initializing {service_name} to fetch '{service}' entities...")

    try:
        gam_client = init_gam_connection(network_code)
        service = gam_client.GetService(service_name)
        fetch_method = getattr(service, method_name)
    except Exception as e:
        print(
            f"Failed to initialize service '{service_name}' or method '{method_name}'."
        )
        raise e

    query_parts = []
    if query_filter:
        query_parts.append(query_filter)

    # Always order by ID for stable and reliable pagination
    query_parts.append("ORDER BY id ASC")

    full_query = " ".join(query_parts)
    statement = ad_manager.FilterStatement(full_query)

    all_items = []
    page_number = 1

    while True:
        print(
            f"Fetching page {page_number} (limit: {gam_api_page_limit}, offset: {statement.offset or 0})..."
        )

        response = fetch_method(statement.ToStatement())

        if response and "results" in response and response["results"]:
            num_results = len(response["results"])
            print(f"-> Found {num_results} items on this page.")

            all_items.extend(response["results"])

            statement.offset += gam_api_page_limit
            page_number += 1

            if num_results < gam_api_page_limit:
                break
        else:
            print("No more items found.")
            break

    print(
        f"Successfully fetched a total of {len(all_items)} '{service}' items.\n"
    )
    all_items_as_dicts = [
        serialize_gam_object(item, columns_to_keep) for item in all_items
    ]

    return pd.DataFrame(all_items_as_dicts)



import pandas as pd
from py2neo import Graph
from py2neo import ClientError

import sroka.config.config as config


def neo4j_query_data(cypher, parameters=None, **kwparameters):

    if type(cypher) != str:
        print('Cypher query needs to be a string')
        return pd.DataFrame([])

    if len(cypher) == 0:
        print('Cypher query cannot be empty')
        return pd.DataFrame([])

    if parameters and type(parameters) != dict:
        print('Parameters need to be a dictionary')
        return pd.DataFrame([])

    neo4j_username = config.get_value('neo4j', 'neo4j_username')
    neo4j_password = config.get_value('neo4j', 'neo4j_password')
    neo4j_address = config.get_value('neo4j', 'neo4j_address')

    secure_graph = Graph("bolt://{}:{}@{}".format(neo4j_username, neo4j_password, neo4j_address))

    try:
        results = secure_graph.run(cypher, parameters, **kwparameters)
    except ClientError as e:
        print('There was an issue with the cypher query')
        print(e)
        return pd.DataFrame([])
    except AttributeError as e:
        print('There was an authentication issue')
        print(e)
        return pd.DataFrame([])

    results_pandas = pd.DataFrame(results.data())

    return results_pandas



import pandas as pd
from botocore.exceptions import ClientError, EndpointConnectionError
from retrying import retry


def input_check(input_to_check, expected_types):
    for expected_type in expected_types:
        if type(input_to_check) == expected_type:
            if expected_type == str and len(input_to_check) == 0:
                print('Function input must be a nonempty string.')
                return False
            return True
    print('Function input must be a string.')
    return False


def return_on_exception(filename):
    return None if filename or filename == '' else pd.DataFrame([])


@retry(stop_max_attempt_number=10,
       wait_exponential_multiplier=1 * 1000,
       wait_exponential_max=10 * 60 * 1000)
def poll_status(session, _id):
    try:
        result = session.get_query_execution(
            QueryExecutionId=_id
        )
    except ClientError as e:
        if e.response['Error']['Code'] == 'InvalidRequestException':
            print("Please check your query_id. Error message:")
        else:
            print("ClientError. Error message:")
        print(e)
        return None
    except EndpointConnectionError as e:
        print('Please check your credentials including aws_region in config.ini file and Internet connection.',
              'Error message:')
        print(e)
        return None

    state = result['QueryExecution']['Status']['State']

    if state in ('SUCCEEDED', 'FAILED', 'CANCELLED'):
        return result
    else:
        print('Waiting for {} query to end'.format(_id))
        raise Exception


def download_file(s3, s3_bucket, s3_key, filename):
    if filename:
        try:
            s3.Bucket(s3_bucket).download_file(s3_key, filename)
        except FileNotFoundError as e:
            print('File or folder not found. Error message:')
            print(e)
            return None
        except ClientError as e:
            if e.response['Error']['Code'] == 'InvalidRequestException':
                print("Please check your query. Error message:")
            else:
                print('Please check your credentials including s3_bucket in config.ini file. Error message:')
            print(e)
            return None
        print('saved to ' + filename)
        return None
    else:
        obj = s3.Object(s3_bucket, s3_key)
        try:
            obj = obj.get()
        except ClientError as e:
            if e.response['Error']['Code'] == 'InvalidRequestException':
                print("Please check your query. Error message:")
            else:
                print('Please check your credentials including s3_bucket in config.ini file. Error message:')
            print(e)
            return None
        try:
            df = pd.read_csv(obj['Body'])
        except ValueError as e:
            print('Something went wrong with query output formatting. Error message:')
            print(e)
            return None
        return df



import sroka.config.config as config
from configparser import NoOptionError


def get_options_from_config():
    # Set the options in a dictionary, in order to pass only the
    # options that were provided in the configuration file to the
    # MySQL connector. Passing empty values would trigger exceptions.
    options = dict()

    try:
        options["host"] = config.get_value('mysql', 'host')
    except (KeyError, NoOptionError):  # Do nothing, this value is optional.
        pass

    try:
        options["port"] = config.get_value('mysql', 'port')
    except (KeyError, NoOptionError):  # Do nothing, this value is optional.
        pass

    try:
        options["user"] = config.get_value('mysql', 'user')
    except (KeyError, NoOptionError):  # Do nothing, this value is optional.
        pass

    try:
        options["password"] = config.get_value('mysql', 'password')
    except (KeyError, NoOptionError):  # Do nothing, this value is optional.
        pass

    try:
        options["unix_socket"] = config.get_value('mysql', 'unix_socket')
    except (KeyError, NoOptionError):  # Do nothing, this value is optional.
        pass

    try:
        options["database"] = config.get_value('mysql', 'database')
    except (KeyError, NoOptionError):  # Do nothing, this value is optional.
        pass

    return options


# Ensures that either the options are set to connect to a remote database
# through a host/port combination, or through a unix socket.
def validate_options(options: dict):
    # If no connection option is given, there is not enough information to
    # connect to the database.
    if "unix_socket" not in options and "host" not in options:
        print('Invalid Configuration: In order to connect to the MySQL database, '
              'the host/port options or the unix_socket option must be set.')
        return False

    # If both keys are present and both have values, the configuration is
    # ambiguous.
    if "unix_socket" in options and "host" in options and \
            options["unix_socket"] != "" and options["host"] != "":
        print('Invalid Configuration: In order to connect to the MySQL database, '
              'the host/port options or the unix_socket option must be set.')
        return False
    return True



import re
import tempfile

import boto3
from qds_sdk.account import Account
from qds_sdk.commands import Command, HiveCommand, PrestoCommand
from qds_sdk.exception import UnauthorizedAccess
from qds_sdk.qubole import Qubole

import sroka.config.config as config
from sroka.api.qubole.qubole_api import execute_with_handling_errors


def _download_to_local(s3, s3_path, fp, delim=None):
    s3_file_pattern = re.compile(r's3://([^/]+)/?(.*)')

    match = s3_file_pattern.match(s3_path)
    bucket_name = match.group(1)

    if s3_path.endswith('/') is False:
        key_name = match.group(2)
        file = s3.Object(bucket_name, key_name)
        fp.write(re.sub(r'\x01', delim, str(file.get()['Body'].read(), 'utf-8')))

    else:
        bucket = s3.Bucket(bucket_name)
        key_prefix = match.group(2)
        for file in bucket.objects.filter(Prefix=key_prefix):
            if 'SUCCESS' not in file.key:
                fp.write(re.sub(r'\x01', delim, str(file.get()['Body'].read(), 'utf-8')))


def _get_results(command, output, delimiter=None):
    conn = Qubole.agent()
    storage_credentials = conn.get(Account.credentials_rest_entity_path)

    session = boto3.Session(
            aws_access_key_id=storage_credentials['storage_access_key'],
            aws_secret_access_key=storage_credentials['storage_secret_key']
        )

    s3 = session.resource('s3')

    result_path = command.meta_data['results_resource']
    results = conn.get(result_path, {'inline': False, 'include_headers': 'false'})
    for s3_path in results['result_location']:
        _download_to_local(s3, s3_path, output, delimiter)


def get(query, delete_file=True, filepath='', delimiter=';', query_type='presto', cluster_label=None):

    with execute_with_handling_errors(config.get_value, 'qubole', 'api_token') as api_token:
        if api_token is None:
            return

    try:
        Qubole.configure(api_token=api_token)
    except UnauthorizedAccess:
        print("Invalid credentials were provided")
        return

    if isinstance(query, int):
        with execute_with_handling_errors(Command().find, id=query) as command:
            if command is None:
                return
    elif query_type == 'presto':
        with execute_with_handling_errors(PrestoCommand.run, query=query, label=cluster_label) as command:
            if command is None:
                return
    elif query_type == 'hive':
        with execute_with_handling_errors(HiveCommand.run, query=query, label=cluster_label) as command:
            if command is None:
                return
    else:
        print('Please verify your input.')
        return

    if filepath != '':
        file = open(filepath, 'w+')
    else:
        file = tempfile.NamedTemporaryFile(mode='w+', delete=delete_file)

    if command.status == 'done':
        _get_results(command, file, delimiter)
        file.seek(0)

        return file
    else:
        raise Exception('Could not retrieve query results (id: %s, status: %s)' % (command.id, command.status))



import configparser
import json
import os
import stat

from google.oauth2.credentials import Credentials
from google_auth_oauthlib.flow import InstalledAppFlow

default_config_filepath = os.path.expanduser('~/.sroka_config/')


def set_google_credentials(authorized_user_file,
                           key_file_location,
                           scope):

    try:
        credentials = Credentials.from_authorized_user_file(authorized_user_file, scopes=[scope])
    except FileNotFoundError:
        flow = InstalledAppFlow.from_client_secrets_file(key_file_location, [scope])
        credentials = flow.run_local_server()
        os.makedirs(os.path.expanduser('~/.cache/'), exist_ok=True)
        with open(authorized_user_file, 'w') as file:
            json.dump({
                'client_id': credentials._client_id,
                'client_secret': credentials._client_secret,
                'refresh_token': credentials._refresh_token
            }, file)
        os.chmod(authorized_user_file, 0o600)
    return credentials


def get_value(group, key):
    """Function that gets configuration from config file."""

    config_path = default_config_filepath + "config.ini"
    if 'CONFIG_FILE_PATH' in os.environ:
        config_path = os.environ['CONFIG_FILE_PATH']

    config = configparser.ConfigParser()
    readable_by_others = stat.S_IRGRP | stat.S_IROTH
    is_protected = not bool(os.stat(config_path).st_mode & readable_by_others)
    if not is_protected:
        raise Exception("Configuration file {config} is not protected, " +
                        "make sure you're the only one allowed to read " +
                        "it by executing `chmod 600 {config}`".format(config=config_path))
    config.read(config_path)
    return config.get(group, key)


def has_value(group, key):
    try:
        get_value(group, key)
        return True
    except (configparser.NoSectionError, configparser.NoOptionError):
        return False


def get_file_path(group):
    """Function that gets configuration files for APIs."""
    if group == 'google_ad_manager':
        return os.environ.get('CONFIG_AD_MANAGER', os.path.join(default_config_filepath, 'ad_manager.json'))
    if group == 'google_analytics':
        return os.environ.get('CONFIG_GOOGLE_ANALYTICS', os.path.join(default_config_filepath, 'client_secrets.json'))
    if group == 'google_drive':
        return os.environ.get('CONFIG_GOOGLE_DRIVE', os.path.join(default_config_filepath, 'credentials.json'))
    if group == 'google_bigquery':
        return os.environ.get('GOOGLE_APPLICATION_CREDENTIALS',
                              os.path.join(default_config_filepath, 'bigquery_credentials.json'))


def setup_env_variables(filepath=None):
    """Function that sets configuration file path."""
    config_filepath = filepath
    if not os.path.exists(config_filepath):
        raise Exception('Config file {} was not found!'.format(config_filepath))
    os.environ['CONFIG_FILE_PATH'] = config_filepath


def setup_admanager_config(filepath=None):
    """Function that sets configuration for Ad Manager file path."""
    config_filepath = filepath
    if not os.path.exists(config_filepath):
        raise Exception('Config file {} was not found!'.format(config_filepath))
    os.environ['CONFIG_AD_MANAGER'] = config_filepath


def setup_client_secret(filepath=None):
    """Function that sets configuration for Google Analytics file path."""
    config_filepath = filepath
    if not os.path.exists(config_filepath):
        raise Exception('Config file {} was not found!'.format(config_filepath))
    os.environ['CONFIG_GOOGLE_ANALYTICS'] = config_filepath


def setup_bigquery_config(filepath=None):
    """Function that sets configuration for BigQuery file path."""
    config_filepath = filepath
    if not os.path.exists(config_filepath):
        raise Exception('Config file {} was not found!'.format(config_filepath))
    os.environ['GOOGLE_APPLICATION_CREDENTIALS'] = config_filepath


def setup_google_sheets_credentials(filepath=None):
    """Function that sets configuration for Google Sheets file path."""
    config_filepath = filepath
    if not os.path.exists(config_filepath):
        raise Exception('Config file {} was not found!'.format(config_filepath))
    os.environ['CONFIG_GOOGLE_DRIVE'] = config_filepath
